@@ -3,7 +3,7 @@
 
 #include <inttypes.h>
 #include <pthread.h>
-#include <html5.h>
+#include <emscripten/html5.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -80,9 +80,9 @@ uint16_t emscripten_atomic_xor_u16(void/*uint16_t*/ *addr, uint16_t val);
 uint32_t emscripten_atomic_xor_u32(void/*uint32_t*/ *addr, uint32_t val);
 uint64_t emscripten_atomic_xor_u64(void/*uint64_t*/ *addr, uint64_t val); // Emulated with locks, very slow!!
 
-int emscripten_futex_wait(void/*uint32_t*/ *addr, uint32_t val, double maxWaitMilliseconds);
-int emscripten_futex_wake(void/*uint32_t*/ *addr, int count);
-int emscripten_futex_wake_or_requeue(void/*uint32_t*/ *addr, int count, void/*uint32_t*/ *addr2, int cmpValue);
+int emscripten_futex_wait(volatile void/*uint32_t*/ *addr, uint32_t val, double maxWaitMilliseconds);
+int emscripten_futex_wake(volatile void/*uint32_t*/ *addr, int count);
+int emscripten_futex_wake_or_requeue(volatile void/*uint32_t*/ *addr, int count, volatile void/*uint32_t*/ *addr2, int cmpValue);
 
 typedef union em_variant_val
 {
